package com.technology.huahai.web.controller.csm;

import com.github.pagehelper.PageHelper;
import com.technology.huahai.common.core.controller.BaseController;
import com.technology.huahai.common.core.controller.domain.AjaxResult;
import com.technology.huahai.common.core.page.TableDataInfo;
import com.technology.huahai.common.utils.AESEncryptionUtil;
import com.technology.huahai.common.utils.StringUtils;
import com.technology.huahai.common.utils.TokenGeneratorUtils;
import com.technology.huahai.web.dto.ScanRecordDto;
import com.technology.huahai.web.vo.*;
import com.technology.huahai.web.service.CsmService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.net.UnknownHostException;
import java.util.List;

/**
 * @author taoyuan
 * @date 2025/05/07
 * 场所码
 */
@RestController
@RequestMapping("/api/technology")
public class CsmController extends BaseController {


    @Autowired
    private CsmService csmService;

    /**
     * 场所码列表分页接口
     * @param technologyQueryVo
     * @return TechnologyListResultVo
     */
    @PostMapping("/list")
    public TableDataInfo<TechnologyListResultVo> list(@RequestBody TechnologyQueryVo technologyQueryVo,HttpServletRequest httpServletRequest) throws Exception {
        String userMessage = TokenGeneratorUtils.getUserMessage(httpServletRequest);
        String message[] = userMessage.split(":");
        technologyQueryVo.setSfzh(message[0]);
        TableDataInfo tableDataInfo = csmService.getList(technologyQueryVo);
        return tableDataInfo;
    }

    /**
     * 场所下拉框接口
     * @param technologyQueryVo
     * @return
     */
    @PostMapping("/dropDownList")
    public TableDataInfo<TechnologyListResultVo> dropDownList(@RequestBody TechnologyQueryVo technologyQueryVo) throws UnknownHostException {
        TableDataInfo tableDataInfo = csmService.dropDownList(technologyQueryVo);
        return tableDataInfo;
    }

    /**
     * 场所码生成接口
     * @param technologyGenerateVo
     * @return
     */
    @PostMapping("/generateTechnology")
    public AjaxResult generateTechnology(@Valid @RequestBody TechnologyGenerateVo technologyGenerateVo, HttpServletRequest httpServletRequest) throws Exception {
        String userMessage = TokenGeneratorUtils.getUserMessage(httpServletRequest);
        String message[] = userMessage.split(":");
        if(StringUtils.isBlank(technologyGenerateVo.getSfzh())){
            technologyGenerateVo.setSfzh(message[0]);
        }
        if(StringUtils.isBlank(technologyGenerateVo.getUserName())){
            technologyGenerateVo.setUserName(message[1]);
        }
        if(StringUtils.isBlank(technologyGenerateVo.getMobile())){
            technologyGenerateVo.setMobile(message[2]);
        }
        technologyGenerateVo.setCreateName(message[1]);
        technologyGenerateVo.setCreateZjhm(message[0]);
        technologyGenerateVo.setCreateLxdh(message[2]);
        technologyGenerateVo.setCxsfzh(AESEncryptionUtil.encrypt(message[0]));
        AjaxResult ajaxResult = csmService.generateTechnology(technologyGenerateVo);
        return ajaxResult;
        /*if(0==ajaxResult.getCode()){
            TechnologyGenerateVo result = new TechnologyGenerateVo();
            result.setTechnologyCode(technologyGenerateVo.getTechnologyCode());
            return AjaxResult.success(result);
        }else{
            return ajaxResult;
        }*/
    }

    /**
     * 场所码申诉/反馈(弃用)
     * @param feedbackVo
     * @return
     */
    @PostMapping("/generateFeedback")
    public AjaxResult generateFeedback(@Valid @RequestBody FeedbackVo feedbackVo, HttpServletRequest httpServletRequest) throws Exception {
        String userMessage = TokenGeneratorUtils.getUserMessage(httpServletRequest);
        String message[] = userMessage.split(":");
        feedbackVo.setFkrsfzh(message[0]);
        feedbackVo.setFkrName(message[1]);
        csmService.generateFeedback(feedbackVo);
        return AjaxResult.success();
    }

    /**
     * 实人认证
     * @param realPersonAuthenticationVo
     * @return
     */
    @PostMapping("/realPersonAuthentication")
    public AjaxResult realPersonAuthentication(@Valid @RequestBody RealPersonAuthenticationVo realPersonAuthenticationVo, HttpServletRequest httpServletRequest) throws Exception {
        String userMessage = TokenGeneratorUtils.getUserMessage(httpServletRequest);
        String message[] = userMessage.split(":");
        realPersonAuthenticationVo.setSfz(message[0]);
        realPersonAuthenticationVo.setXm(message[1]);
        boolean flag = csmService.realPersonAuthentication(realPersonAuthenticationVo);
        if(flag){
            return AjaxResult.success();
        }else {
            return AjaxResult.error();
        }

    }

    /**
     * 保存扫码记录
     * @param scanCodeRecordVo
     * @return
     */
    @PostMapping("/saveScanCodeRecord")
    public AjaxResult saveScanCodeRecord(@Valid @RequestBody ScanCodeRecordVo scanCodeRecordVo, HttpServletRequest httpServletRequest) throws Exception {
        String userMessage = TokenGeneratorUtils.getUserMessage(httpServletRequest);
        String message[] = userMessage.split(":");
        scanCodeRecordVo.setSfzh(message[0]);
        scanCodeRecordVo.setUserName(message[1]);
        scanCodeRecordVo.setMobile(message[2]);
        return csmService.saveScanCodeRecord(scanCodeRecordVo);
    }

    /**
     * 扫码记录查询
     * @param scanRecordQueryVo
     * @return
     */
    /*@PostMapping("/queryScanRecord")
    public AjaxResult queryScanRecord(@Valid @RequestBody ScanRecordQueryVo scanRecordQueryVo) {
        csmService.queryScanRecord(scanRecordQueryVo);
        return AjaxResult.success();
    }*/

    /**
     * 扫码记录查询
     * @param queryScanRecordVo
     * @return
     */
    @PostMapping("/queryScanCodeRecord")
    public TableDataInfo queryScanCodeRecord(@Valid @RequestBody QueryScanRecordVo queryScanRecordVo, HttpServletRequest httpServletRequest) throws Exception {
        PageHelper.startPage(queryScanRecordVo.getPageNum(),queryScanRecordVo.getPageSize());
        String userMessage = TokenGeneratorUtils.getUserMessage(httpServletRequest);
        String message[] = userMessage.split(":");
        queryScanRecordVo.setCxsfzh(AESEncryptionUtil.encrypt(message[0]));
        List<ScanRecordDto> scanRecordDto = csmService.queryScanRecord(queryScanRecordVo);
        return getDataTable(scanRecordDto);
    }

    /**
     * 根据标准地址编码查询地点
     * @param queryAdressVo
     * @return
     */
    @PostMapping("/getAdressByBzdzbm")
    public AjaxResult<TechnologyListResultVo> getAdressByBzdzbm(@Valid @RequestBody QueryAdressVo queryAdressVo){
        AjaxResult ajaxResult = csmService.getAdressByBzdzbm(queryAdressVo);
        return ajaxResult;
    }

    /**
     * 根据标地址查询持有单位
     * @param QueryAdressByDzVo
     * @return
     */
    @PostMapping("/getAddressByBzdzbm")
    public AjaxResult<TechnologyListResultVo> getAdressByDz(@Valid @RequestBody QueryAddressByBzdzbmVo QueryAdressByDzVo){
        AjaxResult ajaxResult = csmService.getAddressByBzdzbm(QueryAdressByDzVo);
        return ajaxResult;
    }

    /**
     * 根据经纬度查询高德地址
     * @param gdQueryDzVo
     * @return
     */
    @PostMapping("/amap/regeo")
    public AjaxResult<String> getAmapRegeo(@Valid @RequestBody GdQueryDzVo gdQueryDzVo){
        String str = csmService.getAmapRegeo(gdQueryDzVo);
        return AjaxResult.success("操作成功",str);
    }

    /**
     * 层级id查询下一级结构
     * @param queryDdLevelVo
     * @return
     */
    @PostMapping("/queryLevelByLevelId")
    public AjaxResult queryLevelByLevelId(@RequestBody QueryDdLevelVo queryDdLevelVo){
        return csmService.queryLevelByLevelId(queryDdLevelVo);
    }

    /**
     * 根据名称模糊查询层级下地址
     * @param queryDdLevelVo
     * @return
     */
    @PostMapping("/queryDdByLevelAndName")
    public TableDataInfo queryDdByLevelAndName(@RequestBody QueryDdLevelVo queryDdLevelVo){
        return csmService.queryDdByLevelAndName(queryDdLevelVo);
    }

    /**
     * 根据场所码Id返回表单
     * @param queryDdLevelVo
     * @return
     */
    @GetMapping("/queryFormByCsmId")
    public TableDataInfo queryFormByCsmId(@RequestBody QueryDdLevelVo queryDdLevelVo){
        return csmService.queryDdByLevelAndName(queryDdLevelVo);
    }


    /**
     * 注销场所码
     * @param cancellationCsmDto
     * @return
     */
    @PostMapping("/cancellationCsm")
    public AjaxResult cancellationCsm(@RequestBody CancellationCsmDto cancellationCsmDto){
        return csmService.cancellationCsm(cancellationCsmDto);
    }


}
