package com.technology.huahai.web.controller.csm;

import com.technology.huahai.common.core.controller.domain.AjaxResult;
import com.technology.huahai.common.utils.StringUtils;
import com.technology.huahai.common.utils.TokenGeneratorUtils;
import com.technology.huahai.web.dto.LdDto;
import com.technology.huahai.web.dto.RzRzrInfoDto;
import com.technology.huahai.web.dto.RzfDto;
import com.technology.huahai.web.service.CsmFormService;
import com.technology.huahai.web.vo.QueryRzfDto;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * 场所码表单控制器
 * @author taoyuan
 * @date 2025/05/07
 * 场所码
 */
@RestController
@RequestMapping("/api/technology/form")
public class CsmFormController {

    @Autowired
    private CsmFormService csmFormService;

    /**
     * 保存日租房信息
     * @param rzfDto
     * @return
     */
    @PostMapping("/saveRzfInfo")
    public AjaxResult saveRzfInfo(@RequestBody RzfDto rzfDto){
        return csmFormService.saveRzfInfo(rzfDto);
    }

    /**
     * 查询日租房信息
     * @param queryRzfDto
     * @return
     */
    @PostMapping("/queryRzfInfo")
    public AjaxResult<List<RzfDto>> queryRzfInfo(@RequestBody QueryRzfDto queryRzfDto){
        return csmFormService.queryRzfInfo(queryRzfDto);
    }

    /**
     * 保存入住人信息
     * @param rzRzrInfoDto
     * @return
     */
    @PostMapping("/saveRzrInfo")
    public AjaxResult saveRzrInfo(@RequestBody RzRzrInfoDto rzRzrInfoDto, HttpServletRequest httpServletRequest) throws Exception {
        String userMessage = TokenGeneratorUtils.getUserMessage(httpServletRequest);
        String message[] = userMessage.split(":");
        if(StringUtils.isBlank(rzRzrInfoDto.getRzrZjhm())){
            rzRzrInfoDto.setRzrZjhm(message[0]);
        }
        if(StringUtils.isBlank(rzRzrInfoDto.getRzrName())){
            rzRzrInfoDto.setRzrName(message[1]);
        }
        if(StringUtils.isBlank(rzRzrInfoDto.getRzrLxdh())){
            rzRzrInfoDto.setRzrLxdh(message[2]);
        }
        rzRzrInfoDto.setCreateName(message[1]);
        rzRzrInfoDto.setCreateZjhm(message[0]);
        rzRzrInfoDto.setCreateLxdh(message[2]);
        return csmFormService.saveRzrInfo(rzRzrInfoDto);
    }

    /**
     * 离店
     * @param ldDto
     * @return
     */
    @PostMapping("/ld")
    public AjaxResult ld(@Validated @RequestBody LdDto ldDto){
        return csmFormService.ld(ldDto);
    }


}
