package com.technology.huahai.web.service.impl;

import com.alibaba.fastjson2.JSONObject;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.JsonNode;
import com.google.zxing.qrcode.decoder.ErrorCorrectionLevel;
import com.technology.huahai.common.constant.RedisKeyConstant;
import com.technology.huahai.common.core.controller.domain.AjaxResult;
import com.technology.huahai.common.core.page.TableDataInfo;
import com.technology.huahai.common.exception.CsmException;
import com.technology.huahai.common.utils.JsonUtils;
import com.technology.huahai.common.utils.RedisUtils;
import com.technology.huahai.common.utils.file.FileUtils;
import com.technology.huahai.common.utils.uuid.IdUtils;
import com.technology.huahai.web.dto.ScanRecordDto;
import com.technology.huahai.web.vo.*;
import com.technology.huahai.web.mapper.coremapper.CsmMapper;
import com.technology.huahai.web.service.CsmService;
import com.technology.huahai.web.zx.ZxService;
import net.glxn.qrgen.core.image.ImageType;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.*;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import org.springframework.util.Base64Utils;
import org.springframework.web.client.RestTemplate;

import java.io.*;
import java.net.UnknownHostException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.util.Base64;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class CsmServiceImpl implements CsmService {

    protected final Logger logger = LoggerFactory.getLogger(this.getClass());

    @Autowired
    private RestTemplate restTemplate;

    @Autowired
    private ZxService zxService;
    @Autowired
    RedisUtils redisUtils;

    @Autowired
    CsmMapper csmMapper;

    @Value("${zx.userId}")
    private String userId;

    @Value("${zx.userKey}")
    private String userKey;

    @Value("${zx.loginUrl}")
    private String loginUrl;

    @Value("${zx.methodkey}")
    private String methodkey;

    @Value("${zx.outinterfaceUrl}")
    private String outinterfaceUrl;

    @Value("${csm.profile}")
    private String csmPath;

    @Value("${zx.ctUrl}")
    private String ctUrl;

    @Value("${zx.csmCtMethodKey}")
    private String csmCtMethodKey;

    @Value("${zx.gdUrl}")
    private String gdUrl;

    @Value("${csm.mode}")
    private String mode;

    @Value("${wx.ewmUrl}")
    private String ewmUrl;

    @Value("${wx.page}")
    private String page;

    @Value("${wx.env_version}")
    private String env_version;


    @Override
    public boolean realPersonAuthentication(RealPersonAuthenticationVo realPersonAuthenticationVo) {
        /*String token = getZxTokenByRedis();
        if(StringUtils.isBlank(token)){
            throw new CsmException("总线token获取失败");
        }else{
            MultiValueMap<String, String> formData = new LinkedMultiValueMap<>();
            formData.add("methodkey", methodkey);
            String postDataJson = JsonUtils.writeValueAsString(realPersonAuthenticationVo);
            formData.add("postDataJson", postDataJson);

            // 4. 构造请求头，指定 Content-Type 为 application/x-www-form-urlencoded
            HttpHeaders headers = new HttpHeaders();
            headers.setContentType(MediaType.APPLICATION_FORM_URLENCODED);
            headers.set("Authorization","Bearer "+token);

            // 5. 构造请求实体（包含头和体）
            HttpEntity<MultiValueMap<String, String>> requestEntity = new HttpEntity<>(formData, headers);
            // 6. 发送请求并获取响应（假设返回的是 String 类型）
            ZxResultDto zxResultDto = restTemplate.postForObject(outinterfaceUrl, requestEntity, ZxResultDto.class);
            logger.info("实人认证结果 {}", zxResultDto);
            if(0== zxResultDto.getStatus()){
                String data = zxResultDto.getData();
                JSONObject jsonObject = JsonUtils.parseJson(data, JSONObject.class);
                if("0".equals(jsonObject.getString("code"))){
                    return true;
                }else{
                    throw new CsmException(jsonObject.getString("msg"));
                }
            }else{
                throw new CsmException(zxResultDto.getMessage());
            }
        }*/
        return true;
    }

    @Override
    @Transactional
    public AjaxResult saveScanCodeRecord(ScanCodeRecordVo scanCodeRecordVo) {
        //本地留存一份只保留扫码时间及姓名及场所码id
        csmMapper.saveScanRecord(scanCodeRecordVo);
        AjaxResult ajaxResult = zxService.cmsEmsInterfaceOperation(scanCodeRecordVo,"saveScanRecord");
        if(0!=ajaxResult.getCode()){
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return ajaxResult;
        }else{
            return ajaxResult;
        }
    }

    @Override
    public TableDataInfo getList(TechnologyQueryVo technologyQueryVo) throws UnknownHostException {
        TableDataInfo tableDataInfo = new TableDataInfo();
        if("dev".equals(mode)){
            // 设置请求头
            HttpHeaders headers = new HttpHeaders();
            headers.setContentType(MediaType.APPLICATION_JSON);

            // 将请求体和请求头组合到 HttpEntity 中
            HttpEntity<TechnologyQueryVo> requestEntity = new HttpEntity<>(technologyQueryVo, headers);

            // 发送 POST 请求
            String url = ctUrl+":8086/csm/getCsmList";
            logger.info("请求地址"+url);
            ResponseEntity<TableDataInfo> response = restTemplate.postForEntity(url, requestEntity, TableDataInfo.class);
            tableDataInfo = response.getBody();
        }else{
            tableDataInfo = zxService.cmsEmsInterfaceQueryList(technologyQueryVo,"getCsmList");
        }
        return tableDataInfo;
    }

    @Override
    public TableDataInfo dropDownList(TechnologyQueryVo technologyQueryVo) throws UnknownHostException {
        TableDataInfo tableDataInfo = new TableDataInfo();
        if("dev".equals(mode)){
            // 设置请求头
            HttpHeaders headers = new HttpHeaders();
            headers.setContentType(MediaType.APPLICATION_JSON);

            // 将请求体和请求头组合到 HttpEntity 中
            HttpEntity<TechnologyQueryVo> requestEntity = new HttpEntity<>(technologyQueryVo, headers);

            // 发送 POST 请求
            String url = ctUrl+":8086/csm/getDdSelect";
            logger.info("请求地址"+url);
            ResponseEntity<TableDataInfo> response = restTemplate.postForEntity(url, requestEntity, TableDataInfo.class);
            tableDataInfo = response.getBody();
            List rows = tableDataInfo.getRows();
            List<TechnologyListResultVo> rowList = JsonUtils.parseJson(JsonUtils.writeValueAsString(rows),new TypeReference<List<TechnologyListResultVo>>() {});
            for(TechnologyListResultVo technologyListResultVo:rowList){
                Object labelName = redisUtils.get(RedisKeyConstant.dictKey+"SCENARIO_LABEL"+"::"+technologyListResultVo.getLabelId());
                if(labelName!=null){
                    technologyListResultVo.setLabelName(labelName.toString());
                }
            }
            tableDataInfo.setRows(rowList);
        }else{
            tableDataInfo = zxService.cmsEmsInterfaceQueryList(technologyQueryVo,"getDdSelect");
            List<TechnologyListResultVo> rows = tableDataInfo.getRows();
            List<TechnologyListResultVo> rowList = JsonUtils.parseJson(JsonUtils.writeValueAsString(rows),new TypeReference<List<TechnologyListResultVo>>() {});
            for(TechnologyListResultVo technologyListResultVo:rowList){
                Object labelName = redisUtils.get(RedisKeyConstant.dictKey+"SCENARIO_LABEL"+"::"+technologyListResultVo.getLabelId());
                if(labelName!=null){
                    technologyListResultVo.setLabelName(labelName.toString());
                }
            }
            tableDataInfo.setRows(rowList);
        }
        return tableDataInfo;
    }

    @Override
    @Transactional
    public AjaxResult generateTechnology(TechnologyGenerateVo technologyGenerateVo) throws UnknownHostException {
        try {
            //外网保存一份
            csmMapper.saveCsmInfo(technologyGenerateVo);
            logger.info("外网场所码信息保存成功");
            String technologyCode = "";
            try {
                technologyCode = getwxacodeunlimit(technologyGenerateVo.getId());
            }catch (Exception e){
                logger.error("获取微信失败token:",e);
                //极有可能是微信token提前过期导致，重试一次
                technologyCode = getwxacodeunlimit(technologyGenerateVo.getId());
                logger.info("重试后获取的场所码为："+technologyCode);
            }
            // 解码Base64字符串
            byte[] imageBytes = Base64.getDecoder().decode(technologyCode);
            String fileName = FileUtils.writeBytesWeekPackage(imageBytes,csmPath,technologyGenerateVo.getId()+".png");
            technologyGenerateVo.setEwmwjdz(fileName);
            technologyGenerateVo.setTechnologyCode(technologyCode);
            AjaxResult ajaxResult = new AjaxResult();
            if("dev".equals(mode)){
                /*// 设置请求头
                HttpHeaders headers = new HttpHeaders();
                headers.setContentType(MediaType.APPLICATION_JSON);

                // 将请求体和请求头组合到 HttpEntity 中
                HttpEntity<TechnologyGenerateVo> requestEntity = new HttpEntity<>(technologyGenerateVo, headers);

                // 发送 POST 请求
                String url = ctUrl+":8086/csm/saveCsm";
                logger.info("请求地址"+url);
                ResponseEntity<AjaxResult> response = restTemplate.postForEntity(url, requestEntity, AjaxResult.class);
                ajaxResult = response.getBody();*/
                ajaxResult.setData(fileName);
            }else{
                ajaxResult = zxService.cmsEmsInterfaceOperation(technologyGenerateVo,"saveCsm");
                if(0!=ajaxResult.getCode()){
                    TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                }
                ajaxResult.setData(fileName);
            }
            return ajaxResult;
        }catch (Exception e){
            logger.error("二维码生成失败:",e);
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return AjaxResult.error("二维码生成失败！");
        }
    }

    @Override
    public AjaxResult getAdressByBzdzbm(QueryAdressVo queryAdressVo) {
        AjaxResult ajaxResult = new AjaxResult();
        if("dev".equals(mode)){
            // 设置请求头
            HttpHeaders headers = new HttpHeaders();
            headers.setContentType(MediaType.APPLICATION_JSON);

            // 将请求体和请求头组合到 HttpEntity 中
            HttpEntity<QueryAdressVo> requestEntity = new HttpEntity<>(queryAdressVo, headers);

            // 发送 POST 请求
            String url = ctUrl+":8086/csm/getAdressByBzdzbm";
            logger.info("请求地址"+url);
            ResponseEntity<AjaxResult> response = restTemplate.postForEntity(url, requestEntity, AjaxResult.class);
            ajaxResult = response.getBody();
        }else{
            ajaxResult = zxService.cmsEmsInterfaceOperation(queryAdressVo,"getAdressByBzdzbm");
            if(0==ajaxResult.getCode()){
                List<TechnologyListResultVo> rowList = JsonUtils.parseJson(JsonUtils.writeValueAsString(ajaxResult.getData()),new TypeReference<List<TechnologyListResultVo>>() {});
                for(TechnologyListResultVo technologyListResultVo:rowList){
                    Object labelName = redisUtils.get(RedisKeyConstant.dictKey+"SCENARIO_LABEL"+"::"+technologyListResultVo.getLabelId());
                    if(labelName!=null){
                        technologyListResultVo.setLabelName(labelName.toString());
                    }
                }
                ajaxResult.setData(rowList);
            }
        }
        return ajaxResult;
    }

    @Override
    public AjaxResult getAddressByBzdzbm(QueryAddressByBzdzbmVo queryAdressByDzVo) {

        AjaxResult ajaxResult = zxService.cmsEmsInterfaceOperation(queryAdressByDzVo,"getAddressByBzdzbm");
        if(0==ajaxResult.getCode()){
            List<TechnologyListResultVo> rowList = JsonUtils.parseJson(JsonUtils.writeValueAsString(ajaxResult.getData()),new TypeReference<List<TechnologyListResultVo>>() {});
            for(TechnologyListResultVo technologyListResultVo:rowList){
                Object labelName = redisUtils.get(RedisKeyConstant.dictKey+"SCENARIO_LABEL"+"::"+technologyListResultVo.getLabelId());
                if(labelName!=null){
                    technologyListResultVo.setLabelName(labelName.toString());
                }
            }
            ajaxResult.setData(rowList);
        }
        return ajaxResult;
    }

    @Override
    public List<ScanRecordDto> queryScanRecord(QueryScanRecordVo queryScanRecordVo) {
        List<ScanRecordDto> scanRecordDto = csmMapper.queryScanRecord(queryScanRecordVo);
        return scanRecordDto;
    }

    @Override
    public AjaxResult cancellationCsm(CancellationCsmDto cancellationCsmDto) {
        return zxService.cmsEmsInterfaceOperation(cancellationCsmDto,"cancellationCsm");
    }

    @Override
    public AjaxResult csmAuthentication(CsmAuthenticationVo csmAuthenticationVo) {
        return zxService.cmsEmsInterfaceOperation(csmAuthenticationVo,"csmAuthentication");
    }


    @Override
    public String getAmapRegeo(GdQueryDzVo gdQueryDzVo) {
        String url = gdUrl + "?key="+gdQueryDzVo.getKey() + "&location="+gdQueryDzVo.getLocation();
        ResponseEntity<JSONObject> responseEntity = restTemplate.getForEntity(url,JSONObject.class);
        JSONObject result = responseEntity.getBody();
        if("1".equals(result.getString("status"))){
            Object res = result.getJSONObject("regeocode").get("formatted_address");
            if(res instanceof String){
                return res.toString();
            }else{
                return "";
            }
        }else {
            throw new CsmException("获取地址失败");
        }
    }

    @Override
    public void generateFeedback(FeedbackVo feedbackVo) {

        AjaxResult ajaxResult = new AjaxResult();
        if("dev".equals(mode)){
            // 设置请求头
            HttpHeaders headers = new HttpHeaders();
            headers.setContentType(MediaType.APPLICATION_JSON);

            // 将请求体和请求头组合到 HttpEntity 中
            HttpEntity<FeedbackVo> requestEntity = new HttpEntity<>(feedbackVo, headers);

            // 发送 POST 请求
            String url = ctUrl+":8086/csm/generateFeedback";
            logger.info("请求地址"+url);
            ResponseEntity<AjaxResult> response = restTemplate.postForEntity(url, requestEntity, AjaxResult.class);
            ajaxResult = response.getBody();
            if(0!=ajaxResult.getCode()){
                throw new CsmException(ajaxResult.getMsg());
            }
        }else{
            ajaxResult = zxService.cmsEmsInterfaceOperation(feedbackVo,"generateFeedback");
            if(0!=ajaxResult.getCode()){
                throw new CsmException(ajaxResult.getMsg());
            }
        }

    }

    @Override
    public AjaxResult queryLevelByLevelId(QueryDdLevelVo queryDdLevelVo) {
        AjaxResult ajaxResult = zxService.cmsEmsInterfaceOperation(queryDdLevelVo,"queryLevelByLevelId");
        return ajaxResult;
    }

    @Override
    public TableDataInfo queryDdByLevelAndName(QueryDdLevelVo queryDdLevelVo) {
        TableDataInfo tableDataInfo = zxService.cmsEmsInterfaceQueryList(queryDdLevelVo,"queryDdByLevelAndName");
        return tableDataInfo;
    }


    /*public int getCsmNum(String bzdzbm) throws UnknownHostException {
        // 设置请求头
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);

        // 将请求体和请求头组合到 HttpEntity 中
        Map<String,String> map = new HashMap<>();
        map.put("bzdzbm",bzdzbm);
        HttpEntity<Map> requestEntity = new HttpEntity<>(map, headers);

        // 发送 POST 请求
        InetAddress ip = InetAddress.getLocalHost();
        String url = ctUrl+":8086/csm/getCsmNum";
        logger.info("请求地址"+url);
        ResponseEntity<AjaxResult> response = restTemplate.postForEntity(url, requestEntity, AjaxResult.class);
        AjaxResult ajaxResult = response.getBody();
        if(200==ajaxResult.getCode()){
            return Integer.valueOf(String.valueOf(ajaxResult.getData()));
        }else{
            throw new CsmException(ajaxResult.getMsg());
        }
    }*/




    public static String getBase64UrlStr(String bzdzbm){
        String url = "https://ywtb.ga.tj.gov.cn/csm/?bzdzbm="+bzdzbm;
        //String url = "https://ywtb.ga.tj.gov.cn/csm?scene=JINANMA00000008";
        String base64Str = null;
        try {
            // 使用 QRGen 创建二维码并保存为 PNG 文件
            //File file = new File(csmPath+bzdzbm+".png");
            File file = new File("D://"+bzdzbm+".png");
            Files.write(file.toPath(),
                    //net.glxn.qrgen.javase.QRCode.from(url).to(ImageType.PNG).stream().toByteArray());
                    net.glxn.qrgen.javase.QRCode.from(url).withSize(400,400)
                            .withErrorCorrection(ErrorCorrectionLevel.H).to(ImageType.PNG).stream().toByteArray());

            System.out.println("二维码已生成：" + file.getAbsolutePath());
            Base64.Encoder base64 = Base64.getEncoder();
            try (FileInputStream fis = new FileInputStream(file);
                 ByteArrayOutputStream bos = new ByteArrayOutputStream()) {
                byte[] b = new byte[1024];
                int n;
                while ((n = fis.read(b)) != -1) {
                    bos.write(b, 0, n);
                }
                base64Str = base64.encodeToString(bos.toByteArray());
            } catch (IOException e) {
                e.printStackTrace();
                throw new CsmException("二维码生成失败");
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new CsmException("二维码生成失败");
        }
        return base64Str;
    }

    /**
     * 获取微信token
     */
    public synchronized String getWxToken(){
        Object o = redisUtils.get(RedisKeyConstant.wxtokenKey);
        logger.info("微信token: {}",o);
        if(null != o){
            JSONObject wxtokenResult = JsonUtils.parseJson(JsonUtils.writeValueAsString(o), JSONObject.class);
            return wxtokenResult.getString("access_token");
        }else{
            /*String url = xcxTokenUrl+"?grant_type="+grant_type+"&appid="+appid+"&secret="+secret;
            // 6. 发送请求并获取响应（假设返回的是 String 类型）
            JSONObject jsonObject = restTemplate.getForObject(url,JSONObject.class);
            long expires = jsonObject.getLong("expires_in")-300;
            // 获取当前时间
            LocalDateTime now = LocalDateTime.now();
            // 加上 7200 秒（即 2 小时）
            LocalDateTime after = now.plus(7200, ChronoUnit.SECONDS);
            // 定义格式化模板：yyyy-MM-dd'T'HH:mm:ss.SS
            DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd'T'HH:mm:ss.SS");
            // 格式化时间
            String refreshTokenTime = after.format(formatter);
            logger.info("微信token获取结果 {}",jsonObject);
            Object access_token = jsonObject.get("access_token");
            if(null == access_token){
                throw new CsmException("获取微信token失败");
            }else{
                String token = String.valueOf(access_token);
                redisUtils.set(RedisKeyConstant.wxtokenKey,jsonObject,expires);
                return token;
            }*/
            logger.info("未获取到微信token");
            throw new CsmException("未获取到微信token");
        }
    }

    /**
     * 获取小程序二维码并转base64
     * @return
     */
    public String getwxacodeunlimit(long id) {
        String csmId = String.format("%08d", id);
        String token = getWxToken();
        String url = ewmUrl+"?access_token="+token;
        Map paramMap = new HashMap<>();
        paramMap.put("page",page);
        paramMap.put("scene","JINANMA"+csmId);
        paramMap.put("check_path",false);
        paramMap.put("env_version",env_version);
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);

        // 将请求体和请求头组合到 HttpEntity 中
        HttpEntity<Map> requestEntity = new HttpEntity<>(paramMap, headers);

        ResponseEntity<byte[]> response = restTemplate.exchange(url, HttpMethod.POST,requestEntity, byte[].class);
        if (response.getStatusCode() == HttpStatus.OK) {
            // 获取响应体中的字节数据
            byte[] result = response.getBody();
            try {
                JsonNode jsonNode = JsonUtils.readTree(result);
                //转换成功说明二维码生成失败，微信方返回了错误信息
                if(jsonNode.has("errcode")){
                    throw new CsmException("微信错误信息："+jsonNode.get("errmsg").asText());
                }
                throw new CsmException("微信生成场所码失败");
            } catch (IOException e) {
                logger.error("二维码转换信息",e);
                //转换失败说明返回的是图片
                // 将字节数组转换为Base64编码字符串
                return Base64Utils.encodeToString(result);
            }
        } else {
            logger.info(new String(response.getBody(), StandardCharsets.UTF_8));
            throw new CsmException("微信二维码生成失败");
        }
    }

    public static void main(String[] args) {
        getBase64UrlStr("8888888");
    }

}
